/*-------------------------------------------------------------------------
 *
 * int1.h
 *	  PostgreSQL C header definitions for INT1
 *
 * Created by Randolf Richardson, 2017-Jul-20 through 2017-Jul-27
 * http://www.randolfrichardson.com/
 *
 * Portions Copyright (c) 2017, Randolf Richardson
 * Portions Copyright (c) 1996-2017, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * IDENTIFICATION
 *	  contrib/int1/int1.h
 *
 * VERSION HISTORY
 *        2017-Aug-02 v1.01 - Added support for GENERATE_SERIES
 *        2017-Jul-27 v1.00 - Initial version
 *
 *-------------------------------------------------------------------------
 */

#ifndef INT1_H
#define INT1_H

#include "fmgr.h"
#include "postgres.h"

PG_MODULE_MAGIC;

/* 8-bit signed integer, fixed-length and passed-by-reference. */
typedef struct INT1 {
    int8 b;
} INT1;

/* Structure for the popular generate_series() function. */
typedef struct {
    INT1 *current;
    INT1 *finish;
    INT1 *step;
} generate_series_int1_fctx;

#define INT1_MAX		127
#define INT1_MIN		-128
#define INT1_STRING_MAX		(sizeof("-128"))

#define SAMESIGN(a,b)		(((a) < 0) == ((b) < 0))

#define DatumGetINT1_P(X)	((INT1*)DatumGetPointer(X))
#define INT1_PGetDatum(X)	PointerGetDatum(X)
#define PG_GETARG_INT1_P(n)	DatumGetINT1_P(PG_GETARG_DATUM(n))
#define PG_RETURN_INT1_P(x)	return INT1_PGetDatum(x)

#define DatumGetINT1(X) ((INT1 *)PG_DETOAST_DATUM(X)) //DatumGetInt8(X)
#define PG_GETARG_INT1(n) DatumGetINT1(n) //DatumGetInt8(PG_GETARG_DATUM(n)) //PG_GETARG_INT8(n)
#define PG_RETURN_INT1(x) PG_RETURN_POINTER(x) //return Int8GetDatum(x) //PG_RETURN_INT8(x)

//#define INT1_GetDatum(X)        ((Datum)SET_1_BYTE(X))
//#define PG_RETURN_INT1(x)	return Int8GetDatum(x)

////#define DatumGetInt8(X)		((int8)GET_1_BYTE(X))  /* XXX */
////#define PG_GETARG_INT1(n)	DatumGetInt8(PG_GETARG_DATUM(n))
////#define INT1_GetDatum(X)	Int8GetDatum(X)
////#define PG_RETURN_INT1(x)	return INT1_GetDatum(x)

// Input/Output subroutines
Datum int1_in(PG_FUNCTION_ARGS);
Datum int1_out(PG_FUNCTION_ARGS);
Datum int1_recv(PG_FUNCTION_ARGS);
Datum int1_send(PG_FUNCTION_ARGS);

// Cast subroutines
Datum int1_cast_from_bool(PG_FUNCTION_ARGS);
  Datum int1_cast_to_bool(PG_FUNCTION_ARGS);
Datum int1_cast_from_bytea(PG_FUNCTION_ARGS);
  Datum int1_cast_to_bytea(PG_FUNCTION_ARGS);
Datum int1_cast_from_char(PG_FUNCTION_ARGS);
  Datum int1_cast_to_char(PG_FUNCTION_ARGS);
Datum int1_cast_from_float4(PG_FUNCTION_ARGS);
  Datum int1_cast_to_float4(PG_FUNCTION_ARGS);
Datum int1_cast_from_float8(PG_FUNCTION_ARGS);
  Datum int1_cast_to_float8(PG_FUNCTION_ARGS);
Datum int1_cast_from_int2(PG_FUNCTION_ARGS);
  Datum int1_cast_to_int2(PG_FUNCTION_ARGS);
Datum int1_cast_from_int4(PG_FUNCTION_ARGS);
  Datum int1_cast_to_int4(PG_FUNCTION_ARGS);
Datum int1_cast_from_int8(PG_FUNCTION_ARGS);
  Datum int1_cast_to_int8(PG_FUNCTION_ARGS);
Datum int1_cast_from_numeric(PG_FUNCTION_ARGS);
  Datum int1_cast_to_numeric(PG_FUNCTION_ARGS);
Datum int1_cast_from_varbit(PG_FUNCTION_ARGS);
  Datum int1_cast_to_varbit(PG_FUNCTION_ARGS);
Datum int1_cast_from_bit(PG_FUNCTION_ARGS);
  Datum int1_cast_to_bit(PG_FUNCTION_ARGS);
//bytea

// Unary Operator subroutines
Datum int1_uminus(PG_FUNCTION_ARGS);
Datum int1_uplus(PG_FUNCTION_ARGS);
Datum int1_uflip(PG_FUNCTION_ARGS);
Datum int1_ureverse(PG_FUNCTION_ARGS);
Datum int1_abs(PG_FUNCTION_ARGS);

// Comparison Operator subroutines
Datum int1_eq(PG_FUNCTION_ARGS);
Datum int1_ne(PG_FUNCTION_ARGS);
Datum int1_lt(PG_FUNCTION_ARGS);
Datum int1_le(PG_FUNCTION_ARGS);
Datum int1_gt(PG_FUNCTION_ARGS);
Datum int1_ge(PG_FUNCTION_ARGS);

// Conversion Operator subroutines
Datum int1_cfrom_int4(PG_FUNCTION_ARGS);

// Bitwise Operator subroutines
Datum int1_and(PG_FUNCTION_ARGS);
Datum int1_or(PG_FUNCTION_ARGS);
Datum int1_xor(PG_FUNCTION_ARGS);
Datum int1_shl(PG_FUNCTION_ARGS);
Datum int1_shr(PG_FUNCTION_ARGS);
Datum int1_shru(PG_FUNCTION_ARGS);

// Mathematical Operator subroutines
Datum int1_plus(PG_FUNCTION_ARGS);
Datum int1_minus(PG_FUNCTION_ARGS);
Datum int1_mul(PG_FUNCTION_ARGS);
Datum int1_div(PG_FUNCTION_ARGS);
Datum int1_mod(PG_FUNCTION_ARGS);

// Utility subroutines
Datum int1_inc(PG_FUNCTION_ARGS);
Datum int1_dec(PG_FUNCTION_ARGS);
Datum int1_larger(PG_FUNCTION_ARGS);
Datum int1_smaller(PG_FUNCTION_ARGS);
Datum to_hex(PG_FUNCTION_ARGS);

// Generate Series subroutines
Datum generate_series(PG_FUNCTION_ARGS);
Datum generate_series_step(PG_FUNCTION_ARGS);

// Index Support subroutines
Datum int1_cmp(PG_FUNCTION_ARGS);
Datum int1_hash(PG_FUNCTION_ARGS);

#endif   /* INT1_H */
